/*
 * Copyright 2013-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.awspring.cloud.sqs;

import org.springframework.core.NestedRuntimeException;
import org.springframework.lang.Nullable;

/**
 * Exception thrown when an executor is provided with an unsupported {@link java.util.concurrent.ThreadFactory}.
 * Currently, for improved performance, a {@link MessageExecutionThreadFactory} is required.
 *
 * @author Tomaz Fernandes
 * @since 3.0
 */
public class UnsupportedThreadFactoryException extends NestedRuntimeException {

	/**
	 * Create an instance with the provided error message.
	 * @param msg the error message.
	 */
	public UnsupportedThreadFactoryException(String msg) {
		super(msg);
	}

	/**
	 * Create an instance with the provided error message and cause.
	 * @param msg the error message.
	 * @param cause the cause.
	 */
	public UnsupportedThreadFactoryException(@Nullable String msg, @Nullable Throwable cause) {
		super(msg, cause);
	}

}
